/* Copyright (c) 2017 timothyjager
   JBC-Soldering-Controller
   MIT License. See LICENSE file for details.

 * \brief   Handles user input in the JBC soldering station.
 * \author  Kaloyan Dimitrov  www.kae-systems.com
 * \date    13 mar 2019

*/
/******************************************************************************/
/*    Types                                                                   */
/******************************************************************************/
struct LED_status_t {
    LED_color_t  color;
           bool  isUpdatePending;
};

struct LED_rgb_pixel_t {
    uint8_t  r;
    uint8_t  g;
    uint8_t  b;
};

/******************************************************************************/
/*    Private global variables                                                */
/******************************************************************************/
static struct LED_status_t  LED_status[LED_COUNT];

/******************************************************************************/
/*    Private global constants                                                */
/******************************************************************************/
static const struct LED_rgb_pixel_t  LED_toRgbPixel[LED_COLOR_NUM] = {
                        //   g,   r,   b
    /* LED_COLOR_OFF    */ { 0,   0,   0  },
    /* LED_COLOR_RED    */ { 0,  20,   0  },
    /* LED_COLOR_BLUE   */ { 0,   0,   20 },
    /* LED_COLOR_GREEN  */ { 20,   0,  0  },
    /* LED_COLOR_ORANGE */ { 20,  35,  0  },
    /* LED_COLOR_CYAN   */ { 20,   0,  30 },
    /* LED_COLOR_WHITE  */ { 20,  20,  20 }
};

NeoPixelBus<NeoRgbFeature, Neo800KbpsMethod> pixels(PixelCount, WS2812_DATA);

/******************************************************************************/
/*    Public operations                                                       */
/******************************************************************************/
void LED_Init(void)
{
    //Neopixel
    pixels.Begin();                               // This initializes the NeoPixel library.
    pixels.SetPixelColor(0, RgbColor(0, 0, 10)); // Moderately bright green color.
    pixels.SetPixelColor(0, RgbColor(1, 0, 10)); // Moderately bright green color.
    pixels.SetPixelColor(0, RgbColor(2, 0, 10)); // Moderately bright green color.
    pixels.SetPixelColor(0, RgbColor(3, 0, 10)); // Moderately bright green color.
    pixels.Show();
}

/*!
 * \brief  Applies the new colors to all LEDs, if requested.
 * \return -
 */
void LED_Task(void)
{
    uint8_t  i;
       bool  doUpdate = false;
    
    for (i = 0; i < LED_COUNT; i++)
    {
        if (LED_status[i].isUpdatePending)
        {
            LED_color_t  color = LED_status[i].color;
            
            pixels.SetPixelColor(i, RgbColor(LED_toRgbPixel[color].r,
                                             LED_toRgbPixel[color].g,
                                             LED_toRgbPixel[color].b));
            
            LED_status[i].isUpdatePending = false;
            doUpdate = true;
        }
    }
    
    
    if (doUpdate)
    {
        pixels.Show();
    }
}

/*!
 * \brief  Sets a new color to requested LED.
 *         The new color is not executed immediately, but in the next call of LED_Task().
 * \param[in] ledId - the ID of LED
 * \param[in] color - new color to set
 * \return    -
 */
void LED_SetColor(LED_id_t  ledId, LED_color_t  color)
{
    if (color != LED_status[ledId].color)
    {
        LED_status[ledId].color = color;
        LED_status[ledId].isUpdatePending = true;
    }
}
